open Ast
open Interpret
open Compile
open Str

type action = Ast | Interpret | Compile | Version | Invalid

(* Read file into string *)
let read_file f =
  let ic = open_in f in
  let n = in_channel_length ic in
  let s = String.create n in
  really_input ic s 0 n;
  close_in ic;
  (s);;

(* Define references used in argument parsing *)
let file_to_open = ref ""
let file_CPP = ref "result.cpp"
let file_LATEX = ref "result.tex"
let action = ref Invalid 

let version =
"   _______  ________  .___  ___.      ___   .___________. __    __
  |   ____||       /  |   \\/   |     /   \\  |           ||  |  |  |
  |  |__   `---/  /   |  \\  /  |    /  ^  \\ `---|  |----`|  |__|  |
  |   __|     /  /    |  |\\/|  |   /  /_\\  \\    |  |     |   __   |
  |  |____   /  /----.|  |  |  |  /  _____  \\   |  |     |  |  |  |
  |_______| /________||__|  |__| /__/     \\__\\  |__|     |__|  |__|

  EZMath v0.1"
;;

(* Search title and author in Latex literal *)
let re_title = Str.regexp "title[' ']*{\\(.*\\)}";;
let re_author = Str.regexp "author[' ']*{\\(.*\\)}";;

let parse_title f =
  try
    ignore(Str.search_forward re_title f 0);
    Str.matched_string f
  with Not_found -> "title{(No Title)}"
;;

let parse_author f =
  try
    ignore(Str.search_forward re_author f 0);
    Str.matched_string f
  with Not_found -> "author{Unknown Author}"
;;

let main =

let speclist = [("-v", Arg.Unit (fun () -> action := Version), "Displays the version information");
("-a", Arg.String (fun file -> file_to_open := file ; action := Ast), "Names file to print Abstract Syntax Tree");
("-i", Arg.String (fun file -> file_to_open := file ; action := Interpret), "Names file for interpreter to run");
("-c", Arg.String (fun file -> file_to_open := file ; action := Compile), "Names file for compiler to compile");
("-o", Arg.Set_string file_CPP, "Names the generated C++ file (only effective in compiler)");
("-l", Arg.Set_string file_LATEX, "Names the final generated latex file (only effective in compiler)");
]
in let usage_msg =
"OVERVIEW: EZMath Compiler
Author: EZMath Team (pc2618, yw2580, sz2425, zc2291)

Options available:"
in Arg.parse speclist (fun arg -> print_endline ("Unknown argument: " ^ arg ^ ". Use -help or --help for more infomation")) usage_msg;
match !action with
(* Defualt action is Invalid, unless specify -v, -a, -i or -c *)
Invalid -> print_endline "-help or --help for more infomation"
| Version -> print_endline version
| _ ->
  let content = read_file !file_to_open in
  let lexbuf = Lexing.from_string ("$$" ^ content) in (* Add leading $$ for scanner *)
  let program = Parser.program Scanner.code lexbuf in
  let program = (List.rev (fst program), List.rev (snd program)) in
  let (title, author) = (parse_title content, parse_author content)
  in
  match !action with
    Ast -> print_string (Ast.program_s program)
  | Interpret -> Interpret.run program (title, author)
  | Compile -> Compile.translate program (title, author, !file_CPP, !file_LATEX)
  | _ -> print_endline "-help or --help for more infomation"

