#!/bin/sh

EZMATH="./../../EZMath"

# Set time limit for all operations
ulimit -t 30

globallog=testall.log
rm -f $globallog
error=0
globalerror=0

keep=0

Usage() {
    echo "Usage: testall.sh [options] [.tex files]"
    echo "-k    Keep intermediate files"
    echo "-h    Print this help"
    exit 1
}

SignalError() {
    if [ $error -eq 0 ] ; then
    	echo "FAILED"
    	error=1
    fi

    echo "  $1"
}

# Compare <outfile> <reffile> <difffile>
# Compares the outfile with reffile.  Differences, if any, written to difffile
Compare() {
    echo diff -b $1 $2 ">" $3 1>&2
    diff -b "$1" "$2" > "$3" 2>&1 || {
        SignalError "$1 differs"
        echo "FAILED $1 differs from $2" 1>&2
    }
}

# Run <args>
# Report the command, run it, and report any errors
Run() {
    echo $* 1>&2
    eval $*
}


Check() {
    error=0
    basename=`echo $1 | sed 's/.*\\///
                             s/.tex//'`
    echo -n "$basename.tex..."
    echo "###### Testing $basename" 1>&2

    generatedfiles="output/${basename}.out"
    referfile="ref/${basename}.out"
    diff="output/${basename}.diff"
    Run "$EZMATH"  "-i" $1 "1>" ${generatedfiles} "2>&1"
    Compare ${generatedfiles} ${referfile} ${diff}

    #latexfile="output/compile/${basename}.tex"
    #referfile="ref/compile/${basename}.tex"
    #diff="output/compile/${basename}.diff"
    #Run "$EZMATH" "-c" $1 "-l" ${latexfile}
    #Run "g++ -std=c++11 result.cpp" &&
    #Run "./a.out" &&
    #Run "rm -f a.out result.cpp" &&
    #Compare ${latexfile} ${referfile} ${diff}

    if [ $error -eq 0 ] ; then
        if [ $keep -eq 0 ] ; then
            rm -f $generatedfiles $latexfile
        fi
        
        echo "OK"
        echo "###### SUCCESS" 1>&2
    else
        echo "###### FAILED" 1>&2
        globalerror=$error
    fi
}

while getopts kdpsh c; do
    case $c in
    k) # Keep intermediate files
        keep=1
        ;;
    h) # Help
        Usage
        ;;
    esac
done

shift `expr $OPTIND - 1`

if [ $# -ge 1 ]
then
    files=$@
else
    # files="tests/fail-*.tex tests/test-*.tex"
    files="src/fail-*.tex"
fi

for file in $files
do
    case $file in
    *test-*)
        Check $file 2>> $globallog
        ;;
    *fail-*)
        Check $file 2>> $globallog
        ;;
    *)
        echo "unknown file type $file"
        globalerror=1
        ;;
    esac
done

exit $globalerror
